@@ -29,8 +29,6 @@
 #include <Akonadi/SelectionProxyModel>
 #include <KCheckableProxyModel>
 #include <KConfigGroup>
-#include <KContacts/Addressee>
-#include <KContacts/ContactGroup>
 #include <KDescendantsProxyModel>
 #include <KLocalizedString>
 #include <KSelectionProxyModel>
@@ -38,7 +36,6 @@
 #include <QBuffer>
 #include <QItemSelectionModel>
 #include <QPointer>
-#include <QSortFilterProxyModel>
 #include <colorproxymodel.h>
 #include <sortedcollectionproxymodel.h>
 
