@@ -15,7 +15,6 @@
 #include <QObject>
 #include <QWindow>
 
-class QWindow;
 class QQuickWindow;
 class QSortFilterProxyModel;
 
@@ -29,9 +28,15 @@ class KalendarApplication : public QObject
 
 public:
     enum Mode {
-        Event,
-        Todo,
-        Contact,
+        Month = 1,
+        Week = 2,
+        ThreeDay = 4,
+        Day = 8,
+        Schedule = 16,
+        Event = Month | Week | ThreeDay | Day | Schedule,
+        Todo = 32,
+        Contact = 64,
+        Mail = 128,
     };
     Q_ENUM(Mode)
 
@@ -63,6 +68,7 @@ Q_SIGNALS:
     void openDayView();
     void openScheduleView();
     void openTodoView();
+    void openMailView();
     void openContactView();
     void openAboutPage();
     void moveViewForwards();
