@@ -1,10 +1,10 @@
 // SPDX-FileCopyrightText: 2021 Carl Schwan <carlschwan@kde.org>
 // SPDX-License-Identifier: LGPL-2.1-or-later
-#include "akonadi/collectionfilterproxymodel.h"
 #include "about.h"
-#include "agentconfiguration.h"
+#include "akonadi/collectionfilterproxymodel.h"
 #include "calendarmanager.h"
 #include "config-kalendar.h"
+#include "filter.h"
 #include "incidencewrapper.h"
 #include "kalendarapplication.h"
 #include "kalendarconfig.h"
@@ -19,8 +19,6 @@
 #include "tagmanager.h"
 #include <Akonadi/AgentFilterProxyModel>
 #include <KAboutData>
-#include <KCalendarCore/MemoryCalendar>
-#include <KCalendarCore/VCalFormat>
 #include <KDBusService>
 #include <KLocalizedContext>
 #include <KLocalizedString>
@@ -51,6 +49,7 @@ int main(int argc, char *argv[])
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #endif
+    QGuiApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
     QApplication app(argc, argv);
     KLocalizedString::setApplicationDomain("kalendar");
     QCoreApplication::setOrganizationName(QStringLiteral("KDE"));
@@ -98,7 +97,6 @@ int main(int argc, char *argv[])
     aboutData.processCommandLine(&parser);
 
     auto config = KalendarConfig::self();
-    AgentConfiguration agentConfiguration;
     auto kalendarApplication = new KalendarApplication;
     kalendarApplication->setCalendar(CalendarManager::instance()->calendar());
 
@@ -120,11 +118,16 @@ int main(int argc, char *argv[])
     auto tagManager = new TagManager(&engine);
     qmlRegisterSingletonInstance("org.kde.kalendar", 1, 0, "Config", config);
     qmlRegisterSingletonInstance("org.kde.kalendar", 1, 0, "CalendarManager", CalendarManager::instance());
-    qmlRegisterSingletonInstance("org.kde.kalendar", 1, 0, "AgentConfiguration", &agentConfiguration);
     qmlRegisterSingletonInstance("org.kde.kalendar", 1, 0, "TagManager", tagManager);
     qmlRegisterSingletonInstance("org.kde.kalendar", 1, 0, "AboutType", new AboutType());
     qmlRegisterSingletonInstance("org.kde.kalendar", 1, 0, "KalendarApplication", kalendarApplication);
 
+    qmlRegisterSingletonType<Filter>("org.kde.kalendar", 1, 0, "Filter", [](QQmlEngine *engine, QJSEngine *scriptEngine) {
+        Q_UNUSED(engine)
+        Q_UNUSED(scriptEngine)
+        return new Filter;
+    });
+
     qmlRegisterType<IncidenceWrapper>("org.kde.kalendar", 1, 0, "IncidenceWrapper");
     qmlRegisterType<AttendeesModel>("org.kde.kalendar", 1, 0, "AttendeesModel");
     qmlRegisterType<MultiDayIncidenceModel>("org.kde.kalendar", 1, 0, "MultiDayIncidenceModel");
