@@ -4,17 +4,14 @@
 #include "hourlyincidencemodel.h"
 #include "kalendar_debug.h"
 #include <QTimeZone>
+#include <cmath>
 
 HourlyIncidenceModel::HourlyIncidenceModel(QObject *parent)
-    : QAbstractItemModel(parent)
+    : QAbstractListModel(parent)
 {
     mRefreshTimer.setSingleShot(true);
-
-    m_config = KalendarConfig::self();
-    QObject::connect(m_config, &KalendarConfig::showSubtodosInCalendarViewsChanged, this, [&]() {
-        beginResetModel();
-        endResetModel();
-    });
+    mRefreshTimer.setInterval(100);
+    mRefreshTimer.callOnTimeout(this, &HourlyIncidenceModel::resetLayoutLines);
 }
 
 QModelIndex HourlyIncidenceModel::index(int row, int column, const QModelIndex &parent) const
@@ -29,11 +26,6 @@ QModelIndex HourlyIncidenceModel::index(int row, int column, const QModelIndex &
     return {};
 }
 
-QModelIndex HourlyIncidenceModel::parent(const QModelIndex &) const
-{
-    return {};
-}
-
 int HourlyIncidenceModel::rowCount(const QModelIndex &parent) const
 {
     // Number of weeks
@@ -43,11 +35,6 @@ int HourlyIncidenceModel::rowCount(const QModelIndex &parent) const
     return 0;
 }
 
-int HourlyIncidenceModel::columnCount(const QModelIndex &) const
-{
-    return 1;
-}
-
 static double getDuration(const QDateTime &start, const QDateTime &end, int periodLength)
 {
     return ((start.secsTo(end) * 1.0) / 60.0) / periodLength;
@@ -81,7 +68,7 @@ QList<QModelIndex> HourlyIncidenceModel::sortedIncidencesFromSourceModel(const Q
             continue;
         }
 
-        if (!m_config->showSubtodosInCalendarViews()
+        if (!m_showSubTodos
             && !srcIdx.data(IncidenceOccurrenceModel::IncidencePtr).value<KCalendarCore::Incidence::Ptr>()->relatedTo().isEmpty()) {
             continue;
         }
@@ -119,8 +106,6 @@ QList<QModelIndex> HourlyIncidenceModel::sortedIncidencesFromSourceModel(const Q
  * The line grouping algorithm then always picks the first incidence,
  * and tries to add more to the same line.
  *
- * We never mix all-day and non-all day, and otherwise try to fit as much as possible
- * on the same line. Same day time-order should be preserved because of the sorting.
  */
 QVariantList HourlyIncidenceModel::layoutLines(const QDateTime &rowStart) const
 {
@@ -191,17 +176,21 @@ QVariantList HourlyIncidenceModel::layoutLines(const QDateTime &rowStart) const
         // We get a start position relative to the number of period spaces there are in a day
         const auto start = ((startDT.time().hour() * 1.0) * (60.0 / mPeriodLength)) + ((startDT.time().minute() * 1.0) / mPeriodLength);
         auto duration = // Give a minimum acceptable height or otherwise have unclickable incidence
-            qMax(getDuration(startDT, idx.data(IncidenceOccurrenceModel::EndTime).toDateTime().toTimeZone(QTimeZone::systemTimeZone()), mPeriodLength), 2.5);
+            qMax(getDuration(startDT, idx.data(IncidenceOccurrenceModel::EndTime).toDateTime().toTimeZone(QTimeZone::systemTimeZone()), mPeriodLength), 1.0);
 
+        // Make sure incidence doesn't extend past the end of the day
         if (start + duration > periodsPerDay) {
-            duration = periodsPerDay - start + 1;
+            duration = periodsPerDay - start;
         }
 
-        const auto startMinutesFromDayStart = (startDT.time().hour() * 60) + startDT.time().minute();
-        const auto endMinutesFromDayStart = qMin((endDT.time().hour() * 60) + endDT.time().minute(), 24 * 60 * 60);
+        const auto realEndMinutesFromDayStart = qMin((endDT.time().hour() * 60) + endDT.time().minute(), 24 * 60 * 60);
+        // Todos likely won't have end date
+        const auto startMinutesFromDayStart =
+            startDT.isValid() ? (startDT.time().hour() * 60) + startDT.time().minute() : qMax(realEndMinutesFromDayStart - mPeriodLength, 0);
+        const auto displayedEndMinutesFromDayStart = floor(startMinutesFromDayStart + (mPeriodLength * duration));
 
         addToResults(idx, start, duration);
-        setTakenSpaces(startMinutesFromDayStart, endMinutesFromDayStart);
+        setTakenSpaces(startMinutesFromDayStart, displayedEndMinutesFromDayStart);
     }
 
     QHash<int, double> takenWidth; // We need this for potential movers
@@ -228,13 +217,21 @@ QVariantList HourlyIncidenceModel::layoutLines(const QDateTime &rowStart) const
         const auto endDT = incidence[QLatin1String("endTime")].toDateTime().toTimeZone(QTimeZone::systemTimeZone()) < rowEnd
             ? incidence[QLatin1String("endTime")].toDateTime().toTimeZone(QTimeZone::systemTimeZone())
             : rowEnd;
+        const auto duration = incidence[QLatin1String("duration")].toDouble();
 
-        const auto endMinutesFromDayStart = qMin((endDT.time().hour() * 60) + endDT.time().minute(), 24 * 60 * 60);
+        // We need a "real" and "displayed" end time for two reasons:
+        // 1. We need the real end minutes to give a fake start time to todos which do not have a start time
+        // 2. We need the displayed end minutes to be able to properly position those incidences which are displayed as longer
+        // than they actually are
+        const auto realEndMinutesFromDayStart = qMin((endDT.time().hour() * 60) + endDT.time().minute(), 24 * 60 * 60);
+        // Todos likely won't have end date
         const auto startMinutesFromDayStart =
-            startDT.isValid() ? (startDT.time().hour() * 60) + startDT.time().minute() : qMax(endMinutesFromDayStart - mPeriodLength, 0);
+            startDT.isValid() ? (startDT.time().hour() * 60) + startDT.time().minute() : qMax(realEndMinutesFromDayStart - mPeriodLength, 0);
+        const int displayedEndMinutesFromDayStart = floor(startMinutesFromDayStart + (mPeriodLength * duration));
 
         // Get max number of incidences that happen at the same time as this
-        for (int i = startMinutesFromDayStart; i < endMinutesFromDayStart; i++) {
+        // (there can be different numbers of concurrent incidences during the time)
+        for (int i = startMinutesFromDayStart; i < displayedEndMinutesFromDayStart; i++) {
             concurrentIncidences = qMax(concurrentIncidences, takenSpaces[i]);
         }
 
@@ -250,8 +247,8 @@ QVariantList HourlyIncidenceModel::layoutLines(const QDateTime &rowStart) const
         // is empty space at the left of the day column.
         double minStartX = 1.0;
 
-        for (int i = startMinutesFromDayStart; i < endMinutesFromDayStart; i++) {
-            // If this is the first incidence that is taken up this minute position, set details
+        for (int i = startMinutesFromDayStart; i < displayedEndMinutesFromDayStart - 1; i++) {
+            // If this is the first incidence that has taken up this minute position, set details
             if (!startX.contains(i)) {
                 takenWidth[i] = widthShare;
                 startX[i] = priorTakenWidthShare;
@@ -269,15 +266,15 @@ QVariantList HourlyIncidenceModel::layoutLines(const QDateTime &rowStart) const
 
         if (minStartX > 0) {
             priorTakenWidthShare = 0;
-            for (int i = startMinutesFromDayStart; i < endMinutesFromDayStart; i++) {
+            for (int i = startMinutesFromDayStart; i < displayedEndMinutesFromDayStart; i++) {
                 startX[i] = 0;
             }
         }
 
         incidence[QLatin1String("priorTakenWidthShare")] = priorTakenWidthShare;
 
-        if (takenSpaces[startMinutesFromDayStart] < takenSpaces[endMinutesFromDayStart - 1] && priorTakenWidthShare > 0) {
-            potentialMovers.append(PotentialMover{incidence, i, startMinutesFromDayStart, endMinutesFromDayStart});
+        if (takenSpaces[startMinutesFromDayStart] < takenSpaces[displayedEndMinutesFromDayStart - 1] && priorTakenWidthShare > 0) {
+            potentialMovers.append(PotentialMover{incidence, i, startMinutesFromDayStart, displayedEndMinutesFromDayStart});
         }
 
         result[i] = incidence;
@@ -300,9 +297,34 @@ QVariantList HourlyIncidenceModel::layoutLines(const QDateTime &rowStart) const
     return result;
 }
 
+void HourlyIncidenceModel::resetLayoutLines()
+{
+    if(mSourceModel->calendar()->isLoading()) {
+        if(!mRefreshTimer.isActive()) {
+            mRefreshTimer.start(100);
+        }
+        return;
+    }
+
+    beginResetModel();
+
+    m_laidOutLines.clear();
+
+    const auto numPeriods = rowCount({});
+    m_laidOutLines.reserve(numPeriods);
+
+    for(int i = 0; i < numPeriods; ++i) {
+        const auto periodStart = mSourceModel->start().addDays(i).startOfDay();
+        const auto periodIncidenceLayout = layoutLines(periodStart);
+        m_laidOutLines.append(periodIncidenceLayout);
+    }
+
+    endResetModel();
+}
+
 QVariant HourlyIncidenceModel::data(const QModelIndex &idx, int role) const
 {
-    if (!hasIndex(idx.row(), idx.column())) {
+    if (!hasIndex(idx.row(), idx.column()) || !mSourceModel || m_laidOutLines.empty()) {
         return {};
     }
     if (!mSourceModel) {
@@ -313,14 +335,14 @@ QVariant HourlyIncidenceModel::data(const QModelIndex &idx, int role) const
     case PeriodStartDateTime:
         return rowStart;
     case Incidences:
-        return layoutLines(rowStart);
+        return m_laidOutLines.at(idx.row());
     default:
         Q_ASSERT(false);
         return {};
     }
 }
 
-IncidenceOccurrenceModel *HourlyIncidenceModel::model()
+IncidenceOccurrenceModel *HourlyIncidenceModel::model() const
 {
     return mSourceModel;
 }
@@ -328,12 +350,16 @@ IncidenceOccurrenceModel *HourlyIncidenceModel::model()
 void HourlyIncidenceModel::setModel(IncidenceOccurrenceModel *model)
 {
     beginResetModel();
+
+    m_laidOutLines.clear();
     mSourceModel = model;
+    Q_EMIT modelChanged();
+
+    endResetModel();
+
     auto resetModel = [this] {
         if (!mRefreshTimer.isActive()) {
-            beginResetModel();
-            endResetModel();
-            mRefreshTimer.start(50);
+            mRefreshTimer.start(100);
         }
     };
     QObject::connect(model, &QAbstractItemModel::dataChanged, this, resetModel);
@@ -342,10 +368,87 @@ void HourlyIncidenceModel::setModel(IncidenceOccurrenceModel *model)
     QObject::connect(model, &QAbstractItemModel::rowsInserted, this, resetModel);
     QObject::connect(model, &QAbstractItemModel::rowsMoved, this, resetModel);
     QObject::connect(model, &QAbstractItemModel::rowsRemoved, this, resetModel);
-    endResetModel();
 }
 
-int HourlyIncidenceModel::periodLength()
+void HourlyIncidenceModel::slotSourceDataChanged(const QModelIndex &upperLeft, const QModelIndex &bottomRight)
+{
+    if(!upperLeft.isValid() || !bottomRight.isValid()) {
+        return;
+    }
+
+    const auto startRow = upperLeft.row();
+    const auto endRow = bottomRight.row();
+
+    scheduleLayoutLinesUpdates(upperLeft.parent(), startRow, endRow);
+}
+
+void HourlyIncidenceModel::scheduleLayoutLinesUpdates(const QModelIndex &sourceIndexParent, const int sourceFirstRow, const int sourceLastRow)
+{
+    if (!mSourceModel) {
+        return;
+    }
+
+    // If we have no existing laid out lines it means we have not done an
+    // initial setup. Go do that, which will provide incidences anyway
+    if (m_laidOutLines.empty()) {
+        resetLayoutLines();
+        return;
+    }
+
+    // Don't bother if we are going for a full reset soon
+    if (mRefreshTimer.isActive()) {
+        return;
+    }
+
+    for (int i = sourceFirstRow; i <= sourceLastRow; ++i) {
+        if(m_linesToUpdate.count() == m_laidOutLines.count()) {
+            break;
+        }
+
+        const auto sourceModelIndex = mSourceModel->index(i, 0, sourceIndexParent);
+        const auto occurrence = sourceModelIndex.data(IncidenceOccurrenceModel::IncidenceOccurrence).value<IncidenceOccurrenceModel::Occurrence>();
+
+        const auto sourceModelStartDate = mSourceModel->start();
+        const auto startDaysFromSourceStart = sourceModelStartDate.daysTo(occurrence.start.date());
+        const auto endDaysFromSourceStart = sourceModelStartDate.daysTo(occurrence.end.date());
+
+        const auto firstPeriodOccurrenceAppears = startDaysFromSourceStart / mPeriodLength;
+        const auto lastPeriodOccurrenceAppears = endDaysFromSourceStart / mPeriodLength;
+
+        if(firstPeriodOccurrenceAppears > m_laidOutLines.count() || lastPeriodOccurrenceAppears < 0) {
+            continue;
+        }
+
+        const auto lastRow = m_laidOutLines.count() - 1;
+        const auto minRow = qMin(qMax(static_cast<int>(firstPeriodOccurrenceAppears), 0), lastRow);
+        const auto maxRow = qMin(static_cast<int>(lastPeriodOccurrenceAppears), lastRow);
+
+        // We set the layout lines scheduled to redo
+        m_linesToUpdate.insert(minRow);
+        m_linesToUpdate.insert(maxRow);
+    }
+
+    // Throttle how often we are changing the lines or the views will be slow
+    if (!m_updateLinesTimer.isActive()) {
+        m_updateLinesTimer.start(100);
+    }
+}
+
+void HourlyIncidenceModel::updateScheduledLayoutLines()
+{
+    for (const auto lineIndex : m_linesToUpdate) {
+        const auto periodStart = mSourceModel->start().addDays(lineIndex).startOfDay();
+        const auto idx = index(lineIndex, 0);
+        const auto laidOutLine = layoutLines(periodStart);
+
+        m_laidOutLines.replace(lineIndex, laidOutLine);
+        Q_EMIT dataChanged(idx, idx);
+    }
+
+    m_linesToUpdate.clear();
+}
+
+int HourlyIncidenceModel::periodLength() const
 {
     return mPeriodLength;
 }
@@ -355,7 +458,7 @@ void HourlyIncidenceModel::setPeriodLength(int periodLength)
     mPeriodLength = periodLength;
 }
 
-HourlyIncidenceModel::Filters HourlyIncidenceModel::filters()
+HourlyIncidenceModel::Filters HourlyIncidenceModel::filters() const
 {
     return m_filters;
 }
@@ -368,6 +471,23 @@ void HourlyIncidenceModel::setFilters(HourlyIncidenceModel::Filters filters)
     endResetModel();
 }
 
+bool HourlyIncidenceModel::showSubTodos() const
+{
+    return m_showSubTodos;
+}
+
+void HourlyIncidenceModel::setShowSubTodos(const bool showSubTodos)
+{
+    if (showSubTodos == m_showSubTodos) {
+        return;
+    }
+
+    m_showSubTodos = showSubTodos;
+    Q_EMIT showSubTodosChanged();
+
+    resetLayoutLines();
+}
+
 QHash<int, QByteArray> HourlyIncidenceModel::roleNames() const
 {
     return {
