@@ -12,7 +12,6 @@
 #include <QList>
 #include <QSharedPointer>
 #include <QTimer>
-#include <kalendarconfig.h>
 
 namespace KCalendarCore
 {
@@ -24,13 +23,14 @@ class Incidence;
  * The "incidences" roles provides a list of lists, where each list represents a visual line,
  * containing a number of events to display.
  */
-class MultiDayIncidenceModel : public QAbstractItemModel
+class MultiDayIncidenceModel : public QAbstractListModel
 {
     Q_OBJECT
     Q_PROPERTY(int periodLength READ periodLength WRITE setPeriodLength NOTIFY periodLengthChanged)
     Q_PROPERTY(MultiDayIncidenceModel::Filters filters READ filters WRITE setFilters NOTIFY filtersChanged)
     Q_PROPERTY(int incidenceCount READ incidenceCount NOTIFY incidenceCountChanged)
     Q_PROPERTY(IncidenceOccurrenceModel *model READ model WRITE setModel NOTIFY modelChanged)
+    Q_PROPERTY(bool showSubTodos READ showSubTodos WRITE setShowSubTodos NOTIFY showSubTodosChanged)
 
 public:
     enum Filter {
@@ -50,42 +50,51 @@ public:
     explicit MultiDayIncidenceModel(QObject *parent = nullptr);
     ~MultiDayIncidenceModel() override = default;
 
-    QModelIndex index(int row, int column, const QModelIndex &parent = {}) const override;
-    QModelIndex parent(const QModelIndex &index) const override;
-
     int rowCount(const QModelIndex &parent) const override;
-    int columnCount(const QModelIndex &parent) const override;
-
     QVariant data(const QModelIndex &index, int role) const override;
-
     QHash<int, QByteArray> roleNames() const override;
 
-    IncidenceOccurrenceModel *model();
-    void setModel(IncidenceOccurrenceModel *model);
+    IncidenceOccurrenceModel *model() const;
     int periodLength() const;
-    void setPeriodLength(int periodLength);
-    MultiDayIncidenceModel::Filters filters();
-    void setFilters(MultiDayIncidenceModel::Filters filters);
-    bool incidencePassesFilter(const QModelIndex &idx) const;
-    Q_INVOKABLE int incidenceCount();
+    MultiDayIncidenceModel::Filters filters() const;
+    bool showSubTodos() const;
+    int incidenceCount() const;
 
 Q_SIGNALS:
     void periodLengthChanged();
     void filtersChanged();
     void incidenceCountChanged();
     void modelChanged();
+    void showSubTodosChanged();
+
+public Q_SLOTS:
+    void setModel(IncidenceOccurrenceModel *model);
+    void setPeriodLength(int periodLength);
+    void setFilters(MultiDayIncidenceModel::Filters filters);
+    void setShowSubTodos(const bool showSubTodos);
 
 protected:
     void setIncidenceCount(int incidenceCount);
 
+private Q_SLOTS:
+    void resetLayoutLines();
+    void slotSourceDataChanged(const QModelIndex &upperLeft, const QModelIndex &bottomRight);
+    void scheduleLayoutLinesUpdates(const QModelIndex &sourceIndexParent, const int sourceFirstRow, const int sourceLastRow);
+    void updateScheduledLayoutLines();
+    bool incidencePassesFilter(const QModelIndex &idx) const;
+
 private:
-    QTimer mRefreshTimer;
     QList<QModelIndex> sortedIncidencesFromSourceModel(const QDate &rowStart) const;
     QVariantList layoutLines(const QDate &rowStart) const;
+
+    QSet<int> m_linesToUpdate;
+    QTimer mRefreshTimer;
+    QTimer m_updateLinesTimer;
     IncidenceOccurrenceModel *mSourceModel{nullptr};
-    int mPeriodLength{7};
+    QVector<QVariantList> m_laidOutLines;
+    int mPeriodLength = 7;
     MultiDayIncidenceModel::Filters m_filters;
-    KalendarConfig *m_config = nullptr;
+    bool m_showSubTodos = true;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(MultiDayIncidenceModel::Filters)
