@@ -659,7 +659,10 @@ int TodoSortFilterProxyModel::comparePriorities(const QModelIndex &left, const Q
     const auto rightTodo = right.data(TodoModel::TodoPtrRole).value<KCalendarCore::Todo::Ptr>();
     Q_ASSERT(leftTodo);
     Q_ASSERT(rightTodo);
-    if (!leftTodo || !rightTodo || leftTodo->priority() == rightTodo->priority()) {
+    // Todos with no priority have a priority of 0 -- push these to list end in ascending order
+    if (m_sortAscending && leftTodo->priority() == 0) {
+        return 1;
+    } else if (!leftTodo || !rightTodo || leftTodo->priority() == rightTodo->priority()) {
         return 0;
     } else if (leftTodo->priority() < rightTodo->priority()) {
         return -1;
