@@ -4,7 +4,6 @@
 #include "tagmanager.h"
 #include "kalendar_debug.h"
 
-#include <QObject>
 #include <Akonadi/TagCreateJob>
 #include <Akonadi/TagDeleteJob>
 #include <Akonadi/TagModifyJob>
@@ -26,7 +25,7 @@ public:
 
         setDynamicSortFilter(true);
         sort(0);
-    };
+    }
 
     QHash<int, QByteArray> roleNames() const override
     {
